package	jp.co.sds_corp.fixconnect.logger;

import	java.util.ArrayList;
import	java.util.List;

import	jp.co.sds_corp.fixconnect.FIXException;
import	jp.co.sds_corp.fixconnect.message.FIXMessage;
import jp.co.sds_corp.util.common.DumpTool;
import jp.co.sds_corp.util.config.IConfiguration;

public abstract class AbstractMessageLogger implements IMessageLogger
{
	DumpTool	dumpTool = new DumpTool();
	ArrayList<String>	excludeList = new ArrayList<String>();

	public void init( IConfiguration config ) throws Exception {
		IConfiguration attr = config.getAttributes();
		String var;
		setDirection( Direction.NONE );
		var = attr.get( "type" );
		if ( var.equals( "BOTH" )) {
			setDirection( Direction.BOTH );
		} else
		if ( var.equals( "RECV" )) {
			setDirection( Direction.RECV );
		} else
		if ( var.equals( "SEND" )) {
			setDirection( Direction.SEND );
		}
		var = attr.get( "dump" );
		if ( var.equals( "true" )) {
			setDump( true );
		}

		List<IConfiguration> list = config.listChild( "exclude" );
		for ( IConfiguration exCfg : list ) {
			String type = exCfg.getAttributes().get( "MsgType", null );
			if ( type != null && type.length() > 0 ) {
				excludeList.add( type );
			}
		}
	}

	public abstract void log( Direction direction, FIXMessage msg )
		throws FIXException;

	public String dump( FIXMessage msg ) throws Exception {
		String ret = dumpTool.dump( msg.getBuffer() );
		return ret;
	}

	public boolean isLogging( FIXMessage msg ) {
		String type = msg.getMsgType();
		if ( excludeList.contains( type )) {
			return false;
		}
		return true;
	}

	Direction	direction;	// ログタイプ
	boolean	dump;	// dumpフラグ

	/**
	* ログタイプを設定します。
	* @param	var		設定するデータ
	*/
	public void setDirection( Direction var ) {
		this.direction = var;
	}
	/**
	* ログタイプを取得します。
	* @return	ログタイプ
	*/
	public Direction getDirection() {
		return direction;
	}

	/**
	* dumpフラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setDump( boolean var ) {
		this.dump = var;
	}
	/**
	* dumpフラグを取得します。
	* @return	dumpフラグ
	*/
	public boolean isDump() {
		return dump;
	}

}
