package	jp.co.sds_corp.fixconnect.message;

import	jp.co.sds_corp.fixconnect.FIXException;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* FIXメッセージを解析します。
*/
public class FIXParser implements IMessageParser
{
	static final CmnLogger logger = CmnLogFactory.getLogger();

	static class TagBuf {
		int id;
		String value;
	}

	public FIXParser() {
	}

	public void init( IConfiguration config ) throws Exception {
	}

	public FIXMessage createMessage( String msgType )
		throws FIXException
	{
		FIXMessage msg = new FIXMessage();
		try {
			LayoutBuilder builder = LayoutBuilder.getInstance();
			builder.createLayout( msg, msgType );
		} catch ( Exception ex ) {
			throw( new FIXException( "create layout error.", ex ));
		}
		return msg;
	}

	public boolean validate( FIXMessage msg ) {
		return true;
	}

	public int getMessageLength( byte[] buf ) {
		int ret = -1;
		int fldStart = 0;
		for ( int pos = 0; pos < buf.length; pos++ ) {
			if ( buf[pos] != FIXMessage.SOH ) {
				continue;
			}
			TagBuf tagBuf = null;
			try {
				tagBuf = parseField( buf, fldStart, pos - fldStart );
			} catch ( FIXException ex ) {
				// エラーのフィールドを含めて長さを返す。
				// parse()時には、Exception
				continue;
			}
			if ( tagBuf.id == Tag.CheckSum.getId() ) {
				ret = pos + 1;
				break;
			}
			fldStart = pos + 1;
		}
		return ret;
	}

	public FIXMessage parse( byte[] buf )
		throws FIXException
	{
		return parse( buf, 0, buf.length );
	}

	public FIXMessage parse( byte[] buf, int start, int length )
		throws FIXException
	{
		String	beginString = null;
		String	bodyLength = null;
		String	msgType = null;
		boolean	init = false;
		LayoutBuilder builder = LayoutBuilder.getInstance();
		FIXMessage ret = new FIXMessage();
		int fldStart = start;
		for ( int pos = start; pos < ( start + length ); pos++ ) {
			if ( buf[pos] != FIXMessage.SOH ) {
				continue;
			}
			if ( pos <= fldStart ) {
				throw( new FIXParseException( "Null Field found.",
						SessionRejectReason.OTHER ));
			}
			if ( !init ) {
				TagBuf tagBuf = parseField( buf, fldStart, pos - fldStart );
				if ( tagBuf.id == Tag.BeginString.getId()) {
					beginString = tagBuf.value;
				} else
				if ( tagBuf.id == Tag.BodyLength.getId()) {
					bodyLength = tagBuf.value;
					ret.setBodyLength( bodyLength );
				} else
				if ( tagBuf.id == Tag.MsgType.getId()) {
					msgType = tagBuf.value;
					// logger.debug( this, "parse as MsgType=[" + msgType + "]" );
					builder.createLayout( ret, msgType );
					ret.setField( Tag.BeginString.getName(), beginString );
					ret.setField( Tag.BodyLength.getName(), bodyLength );
					init = true;
				} else {
					throw( new FIXException( "illegal tag before MsgType."
							+ " id=[" + tagBuf.id + "]" ));
				}
				fldStart = pos + 1;
				continue;
			}

			Field field = parseField( ret, buf, fldStart, pos - fldStart );
			if ( field.getId() == Tag.CheckSum.getId() ) {
				ret.setLength( pos - start + 1 );
				ret.setCheckSum( field.getValue() );
				return ret;
			}
			fldStart = pos + 1;
		}
		return ret;
	}

	TagBuf parseField( byte[] buf, int start, int length )
		throws FIXException
	{
		String field = new String( buf, start, length );
		int eqIdx = field.indexOf( "=" );
		if ( eqIdx < 0 ) {
			throw( new FIXParseException( "'=' not found.",
					SessionRejectReason.OTHER ));
		}
		String tagStr = field.substring( 0, eqIdx );
		String value = field.substring( eqIdx + 1 );
		int id = -1;
		try {
			id = Integer.valueOf( tagStr ).intValue();
		} catch ( Exception ex ) {
			throw( new FIXParseException( "illegal id.[" + tagStr + "]" ));
		}
		TagBuf ret = new TagBuf();
		ret.id = id;
		ret.value = value;
		return ret;
	}

	Field parseField( FIXMessage msg, byte[] buf, int start, int length )
		throws FIXException
	{
		String field = new String( buf, start, length );
		int eqIdx = field.indexOf( "=" );
		if ( eqIdx < 0 ) {
			throw( new FIXParseException( "'=' not found.",
					SessionRejectReason.OTHER ));
		}
		String tagStr = field.substring( 0, eqIdx );
		String value = field.substring( eqIdx + 1 );
		int id = -1;
		try {
			id = Integer.valueOf( tagStr ).intValue();
		} catch ( Exception ex ) {
			throw( new FIXParseException( "illegal tag id.[" + tagStr + "]" ));
		}

		Field ret = msg.getLayout().findFieldById( id );
		if ( ret == null ) {
			throw( new FIXParseException( "unknown tag id.[" + tagStr + "]" ));
		}
		Component component = ret.getComponent();
		if ( component == null ) {
			ret.setValue( value );
		} else if ( ret.isCountField()) {
			ret.setValue( value );
			try {
				int count = Integer.valueOf( value ).intValue();
				component.setCount( count );
			} catch ( Exception ex ) {
				throw( new FIXParseException( "illegal count."
						+ "[" + tagStr + "=" + value + "]" ));
			}
		} else if ( component.isRepeat()) {
			String name = ret.getName();
			for ( int i = 0; i < component.getCount(); i++ ) {
				Field f = component.findField( name, i );
				if ( f.getValue() == null ) {
					f.setValue( value );
					return f;
				}
			}
		}
		return ret;
	}

}
