package jp.co.sds_corp.fixconnect.message;

import java.io.ByteArrayOutputStream;
import java.io.OutputStreamWriter;

import jp.co.sds_corp.test.BasicTestCase;
import jp.co.sds_corp.util.common.DumpTool;
import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
 * Unit test for simple App.
 */
public class MessageTest extends BasicTestCase
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	DumpTool dump = new DumpTool();
	IMessageParser parser;

	public void setUp() throws Exception {
		super.setUp();
		parser = new FIXParser();
		DOMConfiguration doc = DOMConfiguration.findResource( "test.xml" );
		IConfiguration element = doc.getChild("Client").getChild("Parser");
		parser.init( element );
	}

    /**
     */
    public void testSet1() throws Exception {
		FIXMessage msg = parser.createMessage( "A" );
		assertEquals( "MsgType", "A", msg.getField( "MsgType" ) );

//		assertEquals( "BeginString", "FIX.4.4", msg.getField( "BeginString" ) );
		logger.debug( this, "message:\n" + dump.dump( msg.getBytes() ));
    }

    /**
     */
    public void testSet2() throws Exception {
		FIXMessage msg = parser.createMessage( "A" );
		msg.setField( "MsgTypeGrp.RefMsgType", "BBB" );
		msg.setGroupCount( "MsgTypeGrp", 1 );
		assertEquals( "MsgType", "BBB", msg.getField( "MsgTypeGrp.RefMsgType" ) );
    }

    /**
     */
    public void testSet3() throws Exception {
		FIXMessage msg = parser.createMessage( "A" );
		msg.setField( "MsgTypeGrp[0].RefMsgType", "BBB" );
		msg.setField( "MsgTypeGrp[1].RefMsgType", "CC" );
		logger.debug( this, "" + msg );
		assertEquals( "MsgType", "BBB", msg.getField( "MsgTypeGrp[0].RefMsgType" ) );
		assertEquals( "MsgType", "CC", msg.getField( "MsgTypeGrp[1].RefMsgType" ) );
    }

    /**
     */
    public void testSet4() throws Exception {
		FIXMessage msg = parser.createMessage( "A" );
		msg.setField( "MsgTypeGrp[0].RefMsgType", "BBB" );
		msg.setField( "MsgTypeGrp[1].RefMsgType", "CC" );
		msg.setGroupCount( "MsgTypeGrp", 2 );
		logger.debug( this, "" + msg );
		logger.debug( this, "message:\n" + dump.dump( msg.getBytes() ));
		assertEquals( "MsgType", "BBB", msg.getField( "MsgTypeGrp[0].RefMsgType" ) );
		assertEquals( "MsgType", "CC", msg.getField( "MsgTypeGrp[1].RefMsgType" ) );
    }

    /**
     */
    public void testSerialize1() throws Exception {
		FIXMessage msg = parser.createMessage( "A" );
		msg.setField( "MsgTypeGrp.RefMsgType", "BBB" );
		assert( true );
    }

    /**
     */
    public void testParse1() throws Exception {
		FIXMessage msg = parser.createMessage( "A" );
		msg.setField( "MsgTypeGrp[0].RefMsgType", "BBB" );
		msg.setField( "MsgTypeGrp[1].RefMsgType", "CC" );
		msg.setGroupCount( "MsgTypeGrp", 2 );
		byte[] buf = msg.getBytes();

		FIXMessage msg2 = parser.parse( buf );
		logger.debug( this, "" + msg2 );
		assertEquals( "RefMsgType", "BBB", msg2.getField( "MsgTypeGrp[0].RefMsgType" ) );
		assertEquals( "RefMsgType", "CC", msg2.getField( "MsgTypeGrp[1].RefMsgType" ) );
    }

    /**
     */
    public void testParseErr1() throws Exception {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		OutputStreamWriter wr = new OutputStreamWriter( bos );
		wr.write( "8=FIX.4.4" );
		wr.flush();
		bos.write( 1 );
		wr.write( "35=j" );
		wr.flush();
		bos.write( 1 );
		wr.write( "999=???" );
		wr.flush();
		bos.write( 1 );
		wr.write( "10=000" );
		wr.flush();
		bos.write( 1 );

		byte[] buf = bos.toByteArray();
		logger.debug( this, "parse buffer\n" + dump.dump( buf ));
		try {
			FIXMessage msg = parser.parse( buf );
			fail( "parse it!" );
		} catch ( FIXParseException ex ) {
			logger.debug( this, "exception", ex );
		}
    }

    /**
     */
    public void testTelegram1() throws Exception {
		FIXMessage msg = parser.createMessage( "R" );
		msg.setField( "Request[0].Symbol", "USDJPY" );
		logger.debug( this, "" + msg );
		assertEquals( "Request[0].Symbol", "USDJPY", msg.getField( "Request[0].Symbol" ) );
    }

    /**
     */
    public void testTelegram2() throws Exception {
		FIXMessage msg = parser.createMessage( "Z" );
		msg.setField( "QuoteReqID", "AAAA01" );
		logger.debug( this, "" + msg );
		assertEquals( "QuoteReqID", "AAAA01", msg.getField( "QuoteReqID" ) );
    }

    /**
     */
    public void testTelegram3() throws Exception {
		FIXMessage msg = parser.createMessage( "D" );
		msg.setField( "Symbol", "USDJPY" );
		logger.debug( this, "" + msg );
		assertEquals( "Symbol", "USDJPY", msg.getField( "Symbol" ) );
    }

}
